package com.xxp.library.View;

import android.content.Context;
import android.content.res.TypedArray;
import android.text.InputFilter;
import android.text.InputType;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.constraintlayout.widget.ConstraintLayout;

import com.xxp.library.R;
import com.xxp.library.util.xxUtil;
import com.zhy.autolayout.utils.AutoUtils;

import butterknife.BindView;
import butterknife.ButterKnife;

public class DetailMsgLayout extends LinearLayout {


    TextView tv_title, tv_content, tv_contents, tv_necessiry;
    EditText et_content;
    ImageView iv_right;
    TypedArray attrsArray;
    //     0:text,1:edit,2多行text
    public static int TYPE_TEXT = 0;
    public static int TYPE_EDIT = 1;
    public static int TYPE_TEXTS = 2;

    public DetailMsgLayout(Context context) {
        super(context);
        InitView(null);
    }

    public DetailMsgLayout(Context context, AttributeSet attrs) {
        super(context, attrs);
        InitView(attrs);
    }

    public DetailMsgLayout(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);

        InitView(attrs);

    }

    public void InitView(AttributeSet attrs) {
        View view = LayoutInflater.from(getContext()).inflate(R.layout.v_detail_msg, this, true);
        tv_title = view.findViewById(R.id.tv_title);
        tv_content = view.findViewById(R.id.tv_content);
        et_content = view.findViewById(R.id.et_content);
        tv_contents = view.findViewById(R.id.tv_contents);
        tv_necessiry = view.findViewById(R.id.tv_necessity);
        iv_right = view.findViewById(R.id.iv_right);
        if (attrs != null) {
            attrsArray = getContext().obtainStyledAttributes(attrs, R.styleable.DetailMsgLayout);
            tv_title.setText(attrsArray.getString(R.styleable.DetailMsgLayout_titleText));
            if (attrsArray.getBoolean(R.styleable.DetailMsgLayout_isNecessary, false)) {
                tv_necessiry.setVisibility(View.VISIBLE);
            } else {
                tv_necessiry.setVisibility(View.INVISIBLE);
            }
            if (attrsArray.getInteger(R.styleable.DetailMsgLayout_contentType, 0) == 0) {
                tv_content.setVisibility(View.VISIBLE);
                et_content.setVisibility(View.GONE);
                tv_contents.setVisibility(View.GONE);
                tv_content.setText(attrsArray.getString(R.styleable.DetailMsgLayout_contentText));
                tv_content.setTextColor(attrsArray.getColor(R.styleable.DetailMsgLayout_contentTextColor, getResources().getColor(R.color.tv_gray)));
            } else if (attrsArray.getInteger(R.styleable.DetailMsgLayout_contentType, 0) == 1) {
                tv_content.setVisibility(View.GONE);
                et_content.setVisibility(View.VISIBLE);
                tv_contents.setVisibility(View.GONE);
                if (!xxUtil.isEmpty(attrsArray.getString(R.styleable.DetailMsgLayout_contentHint))) {
                    et_content.setHint(attrsArray.getString(R.styleable.DetailMsgLayout_contentHint));
                }

                et_content.setText(attrsArray.getString(R.styleable.DetailMsgLayout_contentText));
                et_content.setTextColor(attrsArray.getColor(R.styleable.DetailMsgLayout_contentTextColor, getResources().getColor(R.color.tv_gray)));
                et_content.setInputType(attrsArray.getInteger(R.styleable.DetailMsgLayout_contentInputType, InputType.TYPE_TEXT_FLAG_CAP_CHARACTERS));


            } else {
                tv_contents.setVisibility(View.VISIBLE);
                et_content.setVisibility(View.GONE);
                tv_content.setVisibility(View.GONE);
                tv_contents.setText(attrsArray.getString(R.styleable.DetailMsgLayout_contentText));
                tv_contents.setTextColor(attrsArray.getColor(R.styleable.DetailMsgLayout_contentTextColor, getResources().getColor(R.color.tv_gray)));
            }
            if (attrsArray.getBoolean(R.styleable.DetailMsgLayout_hasRight, false)) {
                iv_right.setVisibility(View.VISIBLE);
            } else {
                iv_right.setVisibility(View.GONE);
            }
            if (attrsArray.getInteger(R.styleable.DetailMsgLayout_contentTextNumber, 0) != 0) {
                et_content.setFilters(new InputFilter[]{new InputFilter.LengthFilter(attrsArray.getInteger(R.styleable.DetailMsgLayout_contentTextNumber, 0))});
            }


        }


    }


    public void setContentText(String text) {
        if (text == null || text.equals("")) {
            if (tv_content.getVisibility() == View.VISIBLE) {
                tv_content.setText("-");
            }
            if (tv_contents.getVisibility() == View.VISIBLE) {
                tv_contents.setText("-");
            }
            return;
        }

        if (tv_content.getVisibility() == View.VISIBLE) {
            tv_content.setText(text);
        } else if (et_content.getVisibility() == View.VISIBLE) {
            et_content.setText(text);
            et_content.setSelection(text.length());
        } else {
//            tv_contents.
            tv_contents.setText(text);
        }
    }

    public void setTitle(String title) {
        tv_title.setText(title);
    }

    public void setContentType(int contentType) {

        if (contentType == 0) {
            tv_content.setVisibility(View.VISIBLE);
            et_content.setVisibility(View.GONE);
            tv_contents.setVisibility(View.GONE);
        } else if (contentType == 1) {
            tv_content.setVisibility(View.GONE);
            et_content.setVisibility(View.VISIBLE);
            tv_contents.setVisibility(View.GONE);


        } else {
            tv_contents.setVisibility(View.VISIBLE);
            et_content.setVisibility(View.GONE);
            tv_content.setVisibility(View.GONE);
        }
    }

    public String getContentText() {
        if (tv_content.getVisibility() == View.VISIBLE) {
            return tv_content.getText().toString();
        } else if (et_content.getVisibility() == View.VISIBLE) {
            return et_content.getText().toString();
        } else {
            return tv_contents.getText().toString();
        }
    }


}


