package com.xxp.library.httpUtil;

import android.app.Application;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonParseException;
import com.google.gson.JsonPrimitive;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import com.google.gson.JsonSyntaxException;
import com.xxp.library.BuildConfig;

import java.io.IOException;
import java.lang.reflect.Type;
import java.util.Arrays;
import java.util.Collections;
import java.util.concurrent.TimeUnit;

import okhttp3.CipherSuite;
import okhttp3.ConnectionSpec;
import okhttp3.Interceptor;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.TlsVersion;
import retrofit2.Retrofit;
import retrofit2.adapter.rxjava.RxJavaCallAdapterFactory;
import retrofit2.converter.gson.GsonConverterFactory;

public class ApiClient<T> {

    private static ApiClient apiClient;

    public T lp;


    public ApiClient(Class<T> clazz) {
        lp = InitRetrofit(clazz);
    }

    public static <T> ApiClient getInstance(Class<T> clazz) {

        if (apiClient == null) {
            apiClient = new ApiClient<T>(clazz);
        }
        return apiClient;


    }

    public static Gson buildGson() {
        Gson gson = new GsonBuilder()
                .setDateFormat("yyyy-MM-dd HH:mm:ss")
                .serializeNulls()
                .registerTypeAdapter(int.class, new GsonIntegerDefaultAdapter())
                .create();
        return gson;
    }

    public static class GsonIntegerDefaultAdapter implements JsonSerializer<Integer>, JsonDeserializer<Integer> {
        @Override
        public Integer deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context)
                throws JsonParseException {
            try {
                if (json.getAsString().equals("") || json.getAsString().equals("null")) {//定义为int类型,如果后台返回""或者null,则返回0
                    return 0;
                }
            } catch (Exception ignore) {
            }
            try {
                return json.getAsInt();
            } catch (NumberFormatException e) {
                throw new JsonSyntaxException(e);
            }
        }

        @Override
        public JsonElement serialize(Integer src, Type typeOfSrc, JsonSerializationContext context) {
            return new JsonPrimitive(src);
        }
    }


   /* public static <T> T  InitRetrofit(Class<T> clazz) {
        OkHttpClient okHttpClient = new OkHttpClient.Builder().retryOnConnectionFailure(true).connectTimeout(30, TimeUnit.SECONDS)
                //.addNetworkInterceptor()//拦截器addInterceptor:设置应用拦截器，主要用于设置公共参数，头信息，日志拦截等addNetworkInterceptor：设置网络拦截器，主要用于重试或重写
                .build();
        Retrofit retrofit = new Retrofit.Builder().baseUrl(BuildConfig.BASE_URL).client(okHttpClient).
                addConverterFactory(GsonConverterFactory.create(buildGson())).
                addCallAdapterFactory(RxJavaCallAdapterFactory.create()).build();

        return retrofit.create(clazz);
    }*/

    public T InitRetrofit(Class<T> clazz) {
        String BaseUrl = "";

        BaseUrl = BuildConfig.URL;

        ConnectionSpec spec = new ConnectionSpec.Builder(ConnectionSpec.MODERN_TLS)
                .tlsVersions(TlsVersion.TLS_1_1)
                .tlsVersions(TlsVersion.TLS_1_2)
                .cipherSuites(
                        CipherSuite.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,
                        CipherSuite.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
                        CipherSuite.TLS_DHE_RSA_WITH_AES_128_GCM_SHA256)
                .build();
        ConnectionSpec spec1 = new ConnectionSpec.Builder(ConnectionSpec.CLEARTEXT).build();//兼容http接口
        OkHttpClient okHttpClient = new OkHttpClient.Builder().retryOnConnectionFailure(true).connectTimeout(120, TimeUnit.SECONDS).readTimeout(120,TimeUnit.SECONDS)
                .addInterceptor(InterceptorUtil.HeaderInterceptor())
                .addInterceptor(InterceptorUtil.LogInterceptor())//拦截器addInterceptor:设置应用拦截器，主要用于设置公共参数，头信息，日志拦截等addNetworkInterceptor：设置网络拦截器，主要用于重试或重写
                .connectionSpecs(Arrays.asList(spec, spec1))
                .build();
        Retrofit retrofit = new Retrofit.Builder().baseUrl(BaseUrl).client(okHttpClient).
                addConverterFactory(GsonConverterFactory.create(buildGson())).
                addCallAdapterFactory(RxJavaCallAdapterFactory.create())
                .build();

        return retrofit.create(clazz);
    }


}
