package com.xxp.library.util;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.os.Bundle;
import android.util.Log;
import android.widget.Toast;

import com.amap.api.fence.GeoFence;
import com.amap.api.fence.GeoFenceClient;
import com.amap.api.fence.GeoFenceListener;
import com.amap.api.location.AMapLocation;
import com.amap.api.location.AMapLocationListener;
import com.amap.api.location.DPoint;

import java.util.List;

public class LocationPOIUtil {
    public static final String GEOFENCE_BROADCAST_ACTION = "com.xxp.library.location.broadcast";
    Context context;
    GeoFenceReceiver mGeoFenceReceiver;
    GeoFenceClient mGeoFenceClient;
    setPOIStateBack callBack;

    public LocationPOIUtil(Context context) {
        this.context = context;
        mGeoFenceClient = new GeoFenceClient(context.getApplicationContext());
        GeoFenceListener geoFenceListener = new GeoFenceListener() {
            @Override
            public void onGeoFenceCreateFinished(List<GeoFence> list, int i, String s) {
                if (i == GeoFence.ADDGEOFENCE_SUCCESS) {
                    //成功
                } else {

                }
            }
        };


        IntentFilter filter = new IntentFilter(ConnectivityManager.CONNECTIVITY_ACTION);
//        IntentFilter filter = new IntentFilter();
        filter.addAction(GEOFENCE_BROADCAST_ACTION);
        mGeoFenceReceiver = new GeoFenceReceiver();
        context.registerReceiver(mGeoFenceReceiver, filter);
        mGeoFenceClient.createPendingIntent(GEOFENCE_BROADCAST_ACTION);
        mGeoFenceClient.setGeoFenceListener(geoFenceListener);
        mGeoFenceClient.setActivateAction(GeoFenceClient.GEOFENCE_OUT | GeoFenceClient.GEOFENCE_IN);
    }

    public void setPOI(setPOIStateBack callBack, double latitude, double longitude,float radius) {
        this.callBack = callBack;
        DPoint centerPoint = new DPoint();
        //设置中心点纬度
        centerPoint.setLatitude(latitude);
//设置中心点经度
        centerPoint.setLongitude(longitude);
        mGeoFenceClient.addGeoFence(centerPoint, radius, "自有业务Id");

    }

    public void stopLocationPOI() {
        try {
            context.unregisterReceiver(mGeoFenceReceiver);
        } catch (Throwable e) {
        }

        if (null != mGeoFenceClient) {
            mGeoFenceClient.removeGeoFence();
        }
    }

    public interface setPOIStateBack {
        void rePOIState(LocationBean io);
    }

    public class GeoFenceReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getAction().equals(GEOFENCE_BROADCAST_ACTION)) {
                Bundle bundle = intent.getExtras();
                int status = bundle.getInt(GeoFence.BUNDLE_KEY_FENCESTATUS);
                StringBuffer sb = new StringBuffer();
                LocationBean bean=new LocationBean();
                switch (status) {
                    case GeoFence.STATUS_LOCFAIL:
                        sb.append("定位失败");
                        if (callBack != null) {
                          bean.setInLacation(3);
                        }

                        break;
                    case GeoFence.STATUS_IN:
                        sb.append("进入围栏 ");
                        if (callBack != null) {
                            bean.setInLacation(1);
                        }
                        break;
                    case GeoFence.STATUS_OUT:
                        sb.append("离开围栏 ");
                        if (callBack != null) {
                            bean.setInLacation(0);
                        }
                        break;
                    case GeoFence.STATUS_STAYED:
                        sb.append("停留在围栏内 ");
                        break;
                    default:
                        break;
                }
                xxMapUtil.getInstance(context).getLocationMsg(new AMapLocationListener() {
                    @Override
                    public void onLocationChanged(AMapLocation aMapLocation) {
                        bean.setLatitude(aMapLocation.getLatitude());
                        bean.setLongitude(aMapLocation.getLongitude());
                        callBack.rePOIState(bean);
                    }
                });


            }
        }
    }


}
